# MyApplication.py

# OS or low level imports 
import os # operating system module level 2
import subprocess # level 2 -> create threads
import sys

# Standard Imports
# from ModuleName import ObjectName Chapter 11 page 303
from datetime import datetime, timedelta # 2 objects on one line
import csv

# Custom Imports
# importing modules into an application page 116
import Cart # import ModuleName
import Orders as Ord # import ModuleName as Namespace
from DataSets import dataSets
import Shop.Products as Prod # Create an alias
# App.Shop.Categories.Mens.Products


# Globals / Config / etc

# When you run the application from VS Code (IDE's), it will only autosave (by default) the runtime file.

# Planning and executing your application/program/script
def main(): # bootstrap/main runtime/entry point of application

    try:
        # pass
        # using functions for applications structure (we are still using for reusable blocks of code, plus some....)

        # Product List
        Prod.shopProducts() # calling function, not assigning the return from the method

        # method option using return keyword
        # shop_Products = Prod.shopProducts()

        # Get the data for the customer order
        # web page, desktop app, data stream, ...
        firstName, lastName, orderID, orderDate, orderProcessed, ccApproved, subTotal, taxPerc = dataSets() # run input()'s and return 2 args

        # Call customer information
        # new instance of the function<value> in memory
        customerInformation = Ord.customerInformation(firstName, lastName) # var = return<value>
        print(customerInformation)   # print(var)

        # Order Information
        var_invoiceDate, var_dueDate = Ord.orderInformation()
        # Display the datetimes
        print("=== datetime outputs ===")
        print(var_invoiceDate)
        print(var_dueDate)
        print("=========================")

        # Order Details
        Ord.orderDetails()

        # Credit Card Information

        tpl_ccInfo_stored = Cart.ccInfo()  # tuple

        # pass the data thru the api and get the response code. The response code will be a 4th parameter when the collection is returned to the program

        ### convert the tuple with 3 params to a list
        temp_ccInfo = list(tpl_ccInfo_stored)
        
        ### append the 4th param for "approved" or "declined", define the param as "result"
        #### approved, Approved, APPROVED
        #### page 255 string functions
        #### upper(), lower(), title()
        result = "approved"  # result = value sent back from the api

        '''
        if approved:
        elif Approved:  # DRY
        elif APPROVED:  # DRY
        else

        logical operator
        if result == "approved" or result == "Approved" or result == "APPROVED"

        if result >= "approved" or result <= "APPROVED"  APProVed

        switch result: case approved: case Approved: case APPROVED:

        match keyword -> Python 3.10

        '''

        if result.lower() == "approved":

            # when it is approved
            temp_ccInfo.append(result)  # approved or declined from the merchant

            ### covert the list to a tuple ccInfo
            tpl_ccInfo_returned = tuple(temp_ccInfo)

            print(f"Your credit card information was processed: {tpl_ccInfo_returned}.")

            # page 255 numbers
            # 4111254189653256
            # ************3256
            # Ends with 3256
            last_four = str(tpl_ccInfo_returned[0])[-4:]

        else:
            print(f"You credit card was {result}. Please re-enter your credit card information")
            last_four = "Call us for payment options"

        # Customers PDF reciept
        # Chapter 7 FileIO
        # stdout(terminal)
        # change stdout to txt file receipt_orderID.txt
        # Level 2 -> we would add the 2 instances for the string data formats and use them here

        # Data values for the receipt
        coupon, newSubTotal = Cart.subTotal(subTotal) # int, float
        taxAmount = Cart.taxAmount(taxPerc, subTotal)
        cartTotal1 = Cart.cartTotal(subTotal, taxPerc) # 40,7

        # option 2 calculate cart total using local vars
        # structural object in our functional application
        # Don't want to mix structural and functional objects
        cartTotal2 = newSubTotal + taxAmount # 40 + 2.80 = 42.80 !preferred


        # Order ID
        # get the last order id from the csv file and increment by one

        dataSource = "orders_01162025.csv"

        with open(dataSource, 'r', newline='') as file:
            # creates the buffer
            reader = csv.reader(file)

            # reading from the buffer not the file or memory
            orderID = None

            for row in reader: # for x in ys -> x is row ys is the buffer
                
                print(row) # prints ALL rows to memory

                # the last row in memory sent to the buffer
                if row: # check to see if row 1 is not empty (row 0 is the header)
                    orderID = row[0]  # looking for the 0 index position for the row
                    # [5214, 42.52, 01162025, 02142025] -> row len(4) [0:3]
            
            if orderID != 1000:
                orderID = int(orderID) + 1
            
            else:
                orderID = 1000        

        print(f"=== Next Order ID is {orderID}")

        Ord.write_to_csv(orderID, cartTotal1, var_invoiceDate, var_dueDate)

        # Order Receipt
        # PDF output (txt)

        # stdout() setting is currenlty set to terminal (default in VS Code)
        # stdout() and stdin() are set by your system -> sys module

        # Store the original system settings in a config file or variables
        original_stdout = sys.stdout # terminal

        # Set  the stdout() to a FileIO (.txt)
        fileName = f"orderReceipt_{orderID}.txt"

        # Use an absolute path to the Orders directory
        orderPath = "C:\\Users\\Student\\Desktop\\PythonLevel10113\\Orders\\"

        filePath = orderPath + fileName

        print(filePath)

        with open(filePath, 'w') as file:
            sys.stdout = file

            print(f"=== Customer Order ID {orderID} ===") # stdout(txt)
            print("======================")    
            print(f"SubTotal: ${subTotal:.2f}") # $50.00   .1f 50.0  .4f 50.0001
            print(f"Coupon: ${coupon:.2f} ") # 10 $10 $10.00 option2 -> locale module{en/US}
            print(f"Adjusted Subtotal: ${newSubTotal:.2f}") #40.00    
            print(f"Tax Amount: ${taxAmount:.2f} ") # 2.8
            print(f"Cart Total using Cart Module: ${cartTotal1:.2f} ") # option 1
            print(f"Cart Total using local operator: ${cartTotal2:.2f}") # option 2 !preferred
            print(f"Credit Card Approval Status: {result}.")
            print(f"Credit Card Last 4: {last_four}")

        # stdout(terminal) Last step
        sys.stdout = original_stdout

    except ZeroDivisionError as err:
        print(err)

    except Exception as err:
        print(err)

    finally: # optional BUT highly recommended
        print("The application will run this block either pass or fail.")


# if main statement
# attribute name of the object to test the condition of the runtime
'''
objName -> public obj in memory
_objName -> hidden obj in memory
__objName__ -> the objects attr in memory
'''

if __name__ == "__main__":
    print("Welcome to my shopping cart application")
    print("=======================================")
    main()

# EOF